<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.cloudfinance.it/
 * @since      1.0.0
 *
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/admin
 * @author     Cloud Finance srl <supportemail@cloudfinance.it>
 */
class Freeinvoice_Api_Admin
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	private $settings_page_link;
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->settings_page_link = admin_url('admin.php?page=wc-settings&tab=' . $this->plugin_name);
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Freeinvoice_Api_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Freeinvoice_Api_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/freeinvoice-api-admin.css', array(), $this->version, 'all');
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Freeinvoice_Api_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Freeinvoice_Api_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/freeinvoice-api-admin.js', array('jquery'), $this->version, false);
	}
	/**
	 * Register the administration menu for this plugin into the WordPress Dashboard menu.
	 *
	 * @since    1.0.0
	 */

	public function add_plugin_admin_menu()
	{

		/*
		 * Add a settings page for this plugin to the Settings menu.
		 *
		 * NOTE:  Alternative menu locations are available via WordPress administration menu functions.
		 *
		 *        Administration Menus: http://codex.wordpress.org/Administration_Menus
		 *
		 */
		add_options_page('WooCommerce FreeInvoice API Base Options Functions Setup', 'FreeInvoice API Options', 'manage_options', $this->plugin_name, array($this, 'display_plugin_setup_page'));
	}

	/**
	 * Add settings action link to the plugins page.
	 *
	 * @since    1.0.0
	 */

	public function add_action_links($links)
	{
		/*
		 *  Documentation : https://codex.wordpress.org/Plugin_API/Filter_Reference/plugin_action_links_(plugin_file_name)
		 */
		$settings_link = array(
			'<a href="' . $this->settings_page_link . '">' . __('Settings') . '</a>',
			'<a href="'.Freeinvoice_Api_Comm::LINK_BUY_API.'" target="_blank">' . __('Acquisto lotti fattura', 'freeinvoice-api') . '</a>',
		);
		return array_merge($settings_link, $links);
	}
	/** 
	* Add settings action link to the plugins page.
	 *
	 * @since    1.0.0
	 */
	public function register_plugin_links($links, $plugin_file_name) {
		if ($plugin_file_name=== 'freeinvoice-api/freeinvoice-api.php' )  {
			$links[] = '<a href="'.Freeinvoice_Api_Comm::LINK_DOCUMENTATION.'" target="_blank">' . __( 'Documentation' ) . '</a>';
		}
		return $links;
	}

	/**
	 * Render the settings page for this plugin.
	 *
	 * @since    1.0.0
	 */

	public function display_plugin_setup_page()
	{
		include_once('partials/freeinvoice-api-admin-display.php');
	}
	/**
	 * Load dependencies for additional WooCommerce settings
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	public function freeinvoice_add_settings($settings)
	{
		$settings[] = include plugin_dir_path(dirname(__FILE__)) . 'admin/class-freeinvoice-api-wc-settings.php';
		return $settings;
	}
	/**
	 * Display and edit field value on the single order display page
	 *
	 * @since    1.0.0
	 */
	function add_freeinvoice_api_order_fields_box()
	{
		$fi_api_admin = $this;
		add_meta_box(
			'woocommerce-freeinvoice-api-order-fields-box',
			__('FreeInvoice API', 'freeinvoice-api'),
			array($fi_api_admin, 'edit_billing_fields_form'),
			'shop_order',
			'normal',
			'high'
		);
	}
	/**
	 * Render the billing fields for orders.
	 *
	 * @since    1.0.0
	 */

	public function edit_billing_fields_form($post)
	{
		$order = wc_get_order($post->ID);
		$billing = Freeinvoice_Api_Billing::get_billing_fields_for_admin_order($order);

		include_once('partials/freeinvoice-api-edit-billing.php');
	}
	/**
	 * Update the billing fields for orders.
	 *
	 * @since    1.0.0
	 */
	public function update_billing_fields_form($post_id)
	{
		if (!empty($_POST) && check_admin_referer('freeinvoice-api-order-update', 'freeinvoice-api-field-order-update')) {
			
			Freeinvoice_Api_Billing::update_billing_fields_for_admin_order($post_id, $_POST);
		}
	}

	public function error_checking()
	{
		//controllo impostazioni principali
		$a = explode('/', $this->settings_page_link);
		$fi_settings_page = end($a);
		$a = explode('/', $_SERVER['REQUEST_URI']);
		$current_page = end($a);
		if (strpos($current_page, $fi_settings_page) === false) {
			$msg = '&nbsp;<a href="' . $this->settings_page_link . '">' . __('Vai alle impostazioni', 'freeinvoice-api') . '</a>';
			if (!empty(Freeinvoice_Api_Validator::config_errors())) {
				$msg = __('Il plugin di Free Invoice API risulta attivo ma non correttamente configurato.', 'freeinvoice-api') . $msg;
				Freeinvoice_Api_Comm::show_error($msg);
			} else
		if (get_option(Freeinvoice_Api_Billing::DB_PREFIX . 'sandbox_mode', 'yes') === 'yes') {
				$msg = __('Free Invoice API : Sei in sandbox mode. L\'invio delle fatture è simulato.', 'freeinvoice-api') . $msg;
				Freeinvoice_Api_Comm::show_notice($msg, 'warning');
			}
		}
	}


	public function freeinvoice_api_new_order_column_header($columns)
	{
		$new_columns = array();
		foreach ($columns as $column_name => $column_info) {
			$new_columns[$column_name] = $column_info;
			if ('order_status' === $column_name) {
				$new_columns['freeinvoice_api_new_order_column'] = __('FreeInvoice API', 'freeinvoice-api');
			}
		}
		// Freeinvoice_Api_Billing::need_update_reset();
		return $new_columns;
	}

	public function freeinvoice_api_add_order_column_button($column)
	{
		add_thickbox();

		global $post;

		if ('freeinvoice_api_new_order_column' === $column) {
			include('partials/freeinvoice-api-admin-send-invoice.php');
		}
	}
	/**
	 * send invoice the billing fields for orders.
	 *
	 * @since    1.0.0
	 */
	public function send_order_invoice()
	{
		if (isset($_POST['order_id']) && check_ajax_referer('freeinvoice-api-send-order-' . $_POST['order_id']))
			Freeinvoice_Api_Comm::send_order_invoice($_POST['order_id']);
		else
			echo "no admin referer";
		die();
	}
}
