<?php

/**
 * Fired during plugin activation
 *
 * @link       https://www.cloudfinance.it/
 * @since      1.0.0
 *
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/includes
 * @author     Cloud Finance srl <supportemail@cloudfinance.it>
 */
class Freeinvoice_Api_Comm
{
    // generali
    const FREEINVOICE_OPT_API_KEY = Freeinvoice_Api_Billing::DB_PREFIX . 'api_field';

    const SANDBOX_ENDPOINT = 'https://sandbox.freeinvoice.it/api/v1';
    const PRODUCTION_ENDPOINT = 'https://webapp.freeinvoice.it/api/v1';


    const LINK_BUY_API = 'https://webapp.cloudfinance.it/buy/cart-step-cart.php#category-efatture_pack';
    const LINK_DOCUMENTATION = 'https://help.cloudfinance.it/guida-online-category/plugin-freeinvoice-wordpress-woocommerce/';
    const LINK_LOGIN_FREEINVOICE = 'https://webapp.cloudfinance.it';
    const LINK_LOGIN_SANDBOX = 'https://sandbox.freeinvoice.it';

    static $endpoint = self::SANDBOX_ENDPOINT;

    public static function get_freeinvoice_panel_link()
    {
        if (Freeinvoice_Api_Billing::is_sandbox_mode())
            return self::LINK_LOGIN_SANDBOX;
        else return self::LINK_LOGIN_FREEINVOICE;
    }

    // chiamata API a Free Invoice
    public static function api_call($command, $send_data = [], $method = 'POST')
    {
        
        if (Freeinvoice_Api_Billing::is_sandbox_mode())
        self::$endpoint = self::SANDBOX_ENDPOINT;
        else
            self::$endpoint = self::PRODUCTION_ENDPOINT;
        $url = implode('/', array(self::$endpoint, $command));
        if (!isset($send_data['apiKey']))
            $send_data['apiKey'] = get_option(self::FREEINVOICE_OPT_API_KEY);

        $response = wp_remote_post($url, array(
            'method' => $method,
            'timeout' => 45,
            'redirection' => 5,
            'httpversion' => '1.0',
            'blocking' => true,
            'headers' => array(),
            'body' => $send_data,
            'cookies' => array()
        ));

        if (is_wp_error($response)) {
            $error_message = $response->get_error_message();
            _e(self::show_error($error_message));
            $APIStatus = json_encode(array('errors' => [$response, 'code' => 99, 'message' => $error_message]));
            return $APIStatus;
        }
        $body = wp_remote_retrieve_body($response);
    
        return $body;
    }
    public static function send_webhook_endpoint($apiKey)
    {
        return self::add_webhook_endpoint($apiKey);
    }

    private static function add_webhook_endpoint($apiKey, $overwrite_existing = true)
    {
        if ($overwrite_existing) {
            $resp = self::api_call('endpoints', array('apiKey' => $apiKey), 'GET');
            $json = json_decode($resp, true);
            if (isset($json['data']))
                foreach ($json['data'] as $data) {
                    $delete_resp = self::remove_webhook_endpoint($apiKey, $data['id']);
                    if (!isset($delete_resp['data'])) return json_encode($delete_resp);
                }
            else return $resp;
        }
    return self::api_call('endpoints', array('apiKey' => $apiKey, 'url' => home_url('/?webhook=freeinvoice-api', /*'https'*/)));
    }
    private static function remove_webhook_endpoint($apiKey, $id)
    {
        $resp = self::api_call("endpoints/$id", array('apiKey' => $apiKey), 'DELETE');
        return json_decode($resp, true);
    }
    public static function webhook_notification()
    {
        $apiKey = get_option(self::FREEINVOICE_OPT_API_KEY);

        $payload = file_get_contents('php://input');
        $headers = getallheaders();
        $req_signature = $headers['X-Freeinvoice-Signature'];
        $req_timestamp = intval($headers['X-Freeinvoice-Timestamp']);
        $timestamp = time();
        if (abs($timestamp - $req_timestamp) > 300) {
            $msg = ['request timeout' =>  [
                'timestamp_webhook' => $req_timestamp,
                'timestamp_wordpress' => $timestamp
            ]];
            wp_send_json_error(new WP_Error(501, $msg), 501);
        }
        $signature = hash_hmac('sha256', "$req_timestamp.$payload", $apiKey);
        $r = json_decode($payload, true);
        if ($signature === $req_signature) {

            $invoiceId = $r['data']['invoiceId'];
            $new_status = $r['data']['invoiceStatus'];

            $order_id = Freeinvoice_Api_Billing::get_order_id($invoiceId);
            Freeinvoice_Api_Billing::update_invoice_status($order_id, $new_status);
            $msg = ['order_updated' => [
                'order_id' => $order_id,
                'invoice_id' => $invoiceId,
                'new_status' => $new_status
            ]];
            wp_send_json_success($msg);
        } else {
            $msg =  ['invalid signature' => [
                'wordpress' => ['current_timestamp' => $timestamp, 'signature' => $signature, 'apiKey' => $apiKey],
                'request' => ['headers' => $headers, 'payload' => $r]
            ]];
            wp_send_json_error(new WP_Error(502, $msg), 502);
        }
    }

    public static function get_stock_info()
    {
        $resp = Freeinvoice_Api_Comm::api_call('stocks', [], 'GET');
        $json = json_decode($resp);
        if (isset($json->data))
            return $json->data->stocks;
        else throw new Exception("Error Processing Request: no stock data available", 1);
    }

    // 4 Scartata SdI ha risposto all’invio della fattura con una notifica di scarto.
    // La fattura risulta NON emessa e può essere rinviata dopo le
    // correzioni richieste.
    // 5 Elaborazione La fattura è in fase di invio al SdI.
    // Stato temporaneo in attesa di notifiche da parte del SdI.
    // 6 Non inviata -
    // 7 Inviata La fattura è stata inviata al SdI ma non ha ancora ricevuto notifiche
    // di risposta.
    // Stato temporaneo in attesa di notifiche da parte del SdI.
    // 8 Consegnata La fattura è stata correttamente inviata al SdI e consegnata al
    // canale del cessionario committente.
    // La fattura risulta emessa.
    // 9 Non consegnata La fattura è stata correttamente inviata al SdI ma il SdI non è stato
    // in grado di consegnarla al canale del cessionario committente.
    // La fattura risulta emessa
    public static function get_invoice_status($invoiceID)
    {
        $resp = self::api_call("invoices/$invoiceID", array(), 'GET');
        $invoice = json_decode($resp, true);
        if (!isset($invoice['errors']))
            return intval($invoice['data']['invoiceStatus']);
        else return 0;
    }

    public static function send_order_invoice($order_id)
    {
        $order_id = intval($order_id);
        $billing_data = Freeinvoice_Api_Billing::get_billing_data($order_id, $errors);
        if (!empty($errors))
            die();
        extract($billing_data);
        $order = wc_get_order($order_id);
        global $wpdb;
        include(plugin_dir_path(dirname(__FILE__)) . 'includes/freeinvoice-api-template-xml.php');

        $base64 = base64_encode($xml_string);
        $resp = self::api_call('invoices/usend', array('apiKey' => $apiKey, 'invoiceFileBase64' => $base64));

        // echo $base64;

        echo '<div style="display: none">' . $resp . '</div>';
        echo '<a style="display: none" download="invoice_order_' . $order_id . '.xml" href="data:application/xml;base64,' . $base64 . '">Scarica XML</a>';
        $r = json_decode($resp, true);
        if (!isset($r['errors'])) {
            $inv = $r['data'][0];
            Freeinvoice_Api_Billing::update_next_invoice_number($order_id);
            Freeinvoice_Api_Billing::set_invoice_id($order_id, $inv['invoiceId']);
            Freeinvoice_Api_Billing::update_invoice_status($order_id, $inv['invoiceStatus']);
            echo __('Invio riuscito correttamente. Ricarica la pagina.', 'freeinvoice-api');
        } else {
            echo __('Errore nell\'invio della fattura.', 'freeinvoice-api');
        }

    }
    public static function show_notice($msg, $type, $is_dismissible = false)
    {
        add_action('admin_notices', function () use ($msg, $type, $is_dismissible) {
            $class = "notice notice-$type";
            if ($is_dismissible)
                $class .= ' is-dismissible';
            printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), $msg);
        });
    }
    public static function show_error($msg)
    {
        self::show_notice($msg, 'error');
    }
}


class DatiRiepilogo
{
    public $TaxRateId;
    public $AliquotaIVA;
    public $ImponibileImporto;
    public $Imposta;
}
