<?php

/**
 * Fired during plugin activation
 *
 * @link       https://www.cloudfinance.it/
 * @since      1.0.0
 *
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Freeinvoice_Api
 * @subpackage Freeinvoice_Api/includes
 * @author     Cloud Finance srl <supportemail@cloudfinance.it>
 */
class Freeinvoice_Api_Validator
{
    public static function update_order_meta($order_id, $field, $data, $print_error = false)
    {
        update_post_meta($order_id, $field, sanitize_text_field($data));
    }
    private static function check_setting_fields_errors()
    {
        $prefix = Freeinvoice_Api_Billing::DB_PREFIX;
        $err = array();
        $api_key = get_option($prefix . 'api_field', '');
        if (strlen($api_key) < 140)
            $err[$prefix . 'api_field'] = 'API key di Free Invoice non valida, inserisci una key valida.';
        else {
            $e = self::validate_api_key_and_send_webhook_endpoint($api_key);
            if (true!==$e) $err[$prefix . 'api_field'] = $e;
        }
        if (get_option($prefix . 'tax_representative_field', 'denomination') === 'denomination') {
            $denomination = get_option($prefix . 'denomination_field', '');
            $denomination_length = strlen($denomination);
            if ($denomination_length > 80)
                $err[$prefix . 'denomination_field'] = 'L\'agenzia delle entrate non consente una denominazione superiore a 80 caratteri';
            else if ($denomination_length == 0)
                $err[$prefix . 'denomination_field'] = 'Compila il campo della denominazione fiscale oppure seleziona cognome e nome come rappresentante fiscale';
        } else {
            $surname = get_option($prefix . 'surname_field', '');
            $surname_length = strlen($surname);
            if ($surname_length > 60)
                $err[$prefix . 'surname_field'] = 'L\'agenzia delle entrate non consente un cognome superiore a 60 caratteri';
            else if ($surname_length == 0)
                $err[$prefix . 'surname_field'] = 'Compila il campo cognome oppure seleziona denominazione come rappresentante fiscale';
            $name = get_option($prefix . 'name_field', '');
            $name_length = strlen($name);
            if ($name_length > 60)
                $err[$prefix . 'name_field'] = 'L\'agenzia delle entrate non consente un nome superiore a 60 caratteri';
            else if ($name_length == 0)
                $err[$prefix . 'name_field'] = 'Compila il campo nome oppure seleziona denominazione come rappresentante fiscale';
        }
        $vatcode = get_option($prefix . 'vatcode_field', '');
        if (!preg_match('/^[0-9]{11}$/i', $vatcode, $m))
            $err[$prefix . 'vatcode_field'] = 'Partita iva non valida';
        $taxcode = get_option($prefix . 'taxcode_field', '');
        if (!self::is_a_valid_tax_code($taxcode) && $taxcode !== $vatcode)
            $err[$prefix . 'taxcode_field'] = 'Codice fiscale non valido';

        return $err;
    }
    public static function config_errors()
    {
        $prefix = Freeinvoice_Api_Billing::DB_PREFIX;
        $opt_name = $prefix . 'errors';
        $errors =  get_option($opt_name, null);
        if (!isset($errors)) {
            $errors =  self::check_setting_fields_errors();
            add_option($opt_name,$errors);
        }
        return $errors;
    }
    public static function remove_setting_errors()
    {
        $prefix = Freeinvoice_Api_Billing::DB_PREFIX;
        $opt_name = $prefix . 'errors';
        delete_option($opt_name);
    }
    // verifica API key
    public static function validate_api_key_and_send_webhook_endpoint($apiKey = null, $print_error = false)
    {
        $resp = Freeinvoice_Api_Comm::send_webhook_endpoint($apiKey);
        $body = json_decode($resp, true);
        $msg = __('ERROR: connection to Free Invoice API get unusual response, please contact our technical service', 'freeinvoice-api');
        if (!isset($body['data'])) {
            if ($print_error)
                if (isset($body['errors']))
                    foreach ($body['errors'] as $err) {
                        $message = __('ERROR from Free Invoice API server: ', 'freeinvoice-api') . "{$err['message']}(code {$err['code']})";
                        _e(Freeinvoice_Api_Comm::show_error($message));
                    }
                else {
                    _e(Freeinvoice_Api_Comm::show_error($msg));
                }
            else {
                if (isset($body['errors'])) $msg = $body['errors'];
                return $msg;
            }
            return false;
        } else    return true;
    }

    //controllo formale Codice Fiscale (inclusi omocodia)
    public static function is_a_valid_tax_code($code)
    {
        if (!isset($code) || $code == '' || strlen($code) != 16)
            return false;

        if (!preg_match("/[A-Z0-9]+$/", $code))
            return false;
        $s = 0;
        for ($i = 1; $i <= 13; $i += 2) {
            $c = $code[$i];
            if ('0' <= $c and $c <= '9')
                $s += ord($c) - ord('0');
            else
                $s += ord($c) - ord('A');
        }

        for ($i = 0; $i <= 14; $i += 2) {
            $c = $code[$i];
            switch ($c) {
                case '0':
                    $s += 1;
                    break;
                case '1':
                    $s += 0;
                    break;
                case '2':
                    $s += 5;
                    break;
                case '3':
                    $s += 7;
                    break;
                case '4':
                    $s += 9;
                    break;
                case '5':
                    $s += 13;
                    break;
                case '6':
                    $s += 15;
                    break;
                case '7':
                    $s += 17;
                    break;
                case '8':
                    $s += 19;
                    break;
                case '9':
                    $s += 21;
                    break;
                case 'A':
                    $s += 1;
                    break;
                case 'B':
                    $s += 0;
                    break;
                case 'C':
                    $s += 5;
                    break;
                case 'D':
                    $s += 7;
                    break;
                case 'E':
                    $s += 9;
                    break;
                case 'F':
                    $s += 13;
                    break;
                case 'G':
                    $s += 15;
                    break;
                case 'H':
                    $s += 17;
                    break;
                case 'I':
                    $s += 19;
                    break;
                case 'J':
                    $s += 21;
                    break;
                case 'K':
                    $s += 2;
                    break;
                case 'L':
                    $s += 4;
                    break;
                case 'M':
                    $s += 18;
                    break;
                case 'N':
                    $s += 20;
                    break;
                case 'O':
                    $s += 11;
                    break;
                case 'P':
                    $s += 3;
                    break;
                case 'Q':
                    $s += 6;
                    break;
                case 'R':
                    $s += 8;
                    break;
                case 'S':
                    $s += 12;
                    break;
                case 'T':
                    $s += 14;
                    break;
                case 'U':
                    $s += 16;
                    break;
                case 'V':
                    $s += 10;
                    break;
                case 'W':
                    $s += 22;
                    break;
                case 'X':
                    $s += 25;
                    break;
                case 'Y':
                    $s += 24;
                    break;
                case 'Z':
                    $s += 23;
                    break;
            }
        }

        if (chr($s % 26 + ord('A')) != $code[15])
            return false;
        return true;
    }
    public static function validate_italian_vat_code($code)
    {
        $code = strtoupper($code);
        if (preg_match('/^(IT)?([0-9]{11})$/i', $code, $m))
            return end($m);
        else
            return $code; 
    }
}
